# meds/bots/__init__.py
#
#

""" bots unit. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.utils.parse import txt_parse
from meds.utils.time import short_date
from meds.dispatcher import Dispatcher
from meds.main import main, fleet
from meds.defines import BLA, ENDC
from meds.object import Object
from meds.errors import error

from meds.errors import RemoteDisconnect, NotImplemented

import logging
import socket
import time

## BOTS

class Bot(Dispatcher):

    """ Basic Bot. """

    default = ""
    cc = ""

    def __init__(self, *args, **kwargs):
        Dispatcher.__init__(self, *args, **kwargs)
        self._start = time.time()
        self.channels = ["#meds", ]

    ## OUTPUT

    def _raw(self, *args, **kwargs): print(args[0])

    def say(self, *args, **kwargs): self._raw(args[1])
        
    def announce(self, *args, **kwargs):
        """ announce on channels. """
        self._last = time.time()
        logging.info("# %s/announce %s" % (self.type, " ".join(self.channels)))
        for channel in self.channels: self.say(channel, args[0])

    # CHANNELS

    def join(self, *args, **kwargs): pass
        
    def joining(self, *args, **kwargs):
        """ join channels.  """ 
        for channel in self.channels: self.join(channel)

    ## LOOP

    def start(self, *args, **kwargs):
        logging.warn("# %s/start" % self.type)
        self._status = "ok"
        if "collect" in kwargs: logging.warn("# %s/enable collect" % self.type)
        while self._status:
            try:
                event = self.event()
                if not self._status: break
                if not event: continue
                if "txt" in event: event.parsed = txt_parse(self, event.txt)
                else: event.parsed = txt_parse(self, "")
                thrs = self.dispatch(event)
                if thrs and kwargs.get("collect", False):
                    result = self.collect(thrs)
                    logging.info("%s/collect %s" % (self.type, ", ".join([str(x) for x in result])))
            except RemoteDisconnect: time.sleep(5.0) ; continue
            except (EOFError, KeyboardInterrupt): raise
            except: error()
        self.ready()
        logging.warn("# stop %s/%s" % self.type, event.cmnd)

    def event(self, *args, **kwargs):
        event = Object()
        event._target = self
        event.origin = "root@self"
        event.txt = args[0]
        return event

    ## DISPATCH

    def single(self, *args, **kwargs):
        if not args[0]: return
        event = Object()
        event._target = self
        event.origin = "meds@shell"
        event.txt = args[0]
        event.parsed = txt_parse(self, event.txt)
        return self.dispatch(event)

    def dispatch(self, *args, **kwargs):
        event = args[0]
        if "txt" not in event: return
        if "parsed" not in event: event.parsed = txt_parse(self, event.txt)
        cmnds = self.find_cmnd(event.parsed.cmnd)
        thrs = []
        for cmnd in cmnds:
            try: o = self.put(cmnd.func, event) ; thrs.append(o)
            except AttributeError: continue
            except: error()
        return thrs

    def find_cmnd(self, *args, **kwargs):
        from meds.main import main
        cmnd = args[0]
        result = []
        cmnds = main.find(cmnd)
        if not cmnds:
            try: cmnds = main[cmnd]
            except KeyError: cmnds = []
        return cmnds

            
## TESTBOT

class TestBot(Bot):

    """ USE WITH Bot.put() and Bot.read(). """

    testing = True
    cc = "!"
    default = ""

    def _raw(self, *args, **kwargs): print(args[0])

    def say(self, *args, **kwargs): logging.warn(args[1])

    def event(self, *args, **kwargs):
        o = Object()
        o.origin = "test@meds"
        o._target = self
        o.loglevel = "warning"
        o.txt = args[0]
        o.parsed = txt_parse(self, o.txt)
        return o

## CMNDS

def join(event):
    if not event.parsed.args: return
    channel = event.parsed.args[0]
    for bot in fleet:
        if "join" not in bot: continue
        bot.join(channel)
        event.reply("join %s (%s)" % (channel, bot.server))

main.register("bot.join", join)

def part(event):
    if not event.parsed.args: return
    for bot in fleet:
        try: bot.part(event.parsed.args[0])
        except: continue
        event.reply("part %s (%s)" % (channel, bot.server))

main.register("bot.part", part)

def nick(event):
    if not event.parsed.args: return
    for bot in fleet:
        try: bot.nick(event.parsed.args[0])
        except: continue
        event.reply("nick %s" % bot.server)

main.register("bot.nick", nick)

def quit(event):
    if not event.parsed.args: return
    for bot in fleet:
        try: bot.quit(event.parsed.rest)
        except: continue
        event.reply("quit %s" % bot.server)
    
main.register("bot.quit", quit)
