# meds/bots/xmpp.py
#
#

""" basic package for the program. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.cfg import cfg as maincfg
from meds.main import main, fleet
from meds.object import Object
from meds.bots import Bot

import threading
import logging
import getpass
import queue

## XMPP

class XMPP(Bot):

    """ XMPP Bot. """

    cc = "none"
    default = ""

    def __init__(self, password, *args, **kwargs):
        import sleekxmpp
        from sleekxmpp import clientxmpp
        Bot.__init__(self, *args, **kwargs)
        if "port" not in self: self.port = maincfg.xmpp.port
        self._queue = queue.Queue()
        self.client = clientxmpp.ClientXMPP(self.user, password)
        self.add_handler("session_start", self.session_start)
        self.add_handler("message", self.messaged)
        self.add_handler('disconnected', self.disconnected)
        self.add_handler('connected', self.connected)
        self.add_handler('presence_available', self.presenced)
        self.add_handler('presence_dnd', self.presenced)
        self.add_handler('presence_xa', self.presenced)
        self.add_handler('presence_chat', self.presenced)
        self.add_handler('presence_away', self.presenced)
        self.add_handler('presence_unavailable', self.presenced)
        self.add_handler('presence_subscribe', self.presenced)
        self.add_handler('presence_subscribed', self.presenced)
        self.add_handler('presence_unsubscribe', self.presenced)
        self.add_handler('presence_unsubscribed', self.presenced)
        self.add_handler('groupchat_message', self.messaged)
        self.add_handler('groupchat_presence', self.presenced)
        self.add_handler('groupchat_subject', self.presenced)
        self.add_handler('failed_auth', self.failedauth)
        self.client.exception = self.exception
        self.client.use_signals()
        self._connected = threading.Event()
        self.channels = []

    def add_handler(self, key, value):
        logging.info("# %s/add %s" % (self.type, key))
        self.client.add_event_handler(key, value)

    def _raw(self, *args, **kwargs):
        logging.debug("> %s/write %s" % (self.type, args[0]))
        self.client.send_raw(args[0])

    def start(self, *args, **kwargs):
        try: self.connect()
        except: raise
        self.put(Bot.start, self)
        self.loop()

    def connect(self):
        logging.warn("# %s/connect %s" % (self.type, self.user))
        if maincfg.main.openfire:
            logging.warn("# %s/openfire" % self.type)
            import ssl
            self.client.ssl_version = ssl.PROTOCOL_SSLv3
            self.client.connect((self.server, self.port), use_ssl=True)
        else: self.client.connect((self.server, self.port), reattempt=True)

    def session_start(self, event):
        self.client.send_presence()
        self.ready()
        logging.warn("# %s/session %s" % (self.type, self.user))
         
    def exception(self, ex):
        self._error = ex
        logging.error("^ %s/error %s"% (self.type, str(ex)))

    def announce(self, *args, **kwargs):
        for channel in self.channels: self.say(channel, args[0])

    def failedauth(self, error, *args, **kwargs): self._error = error ; logging.error("^ %s/auth %s" % (self.type, str(error)))

    def failure(self, error, *args, **kwargs): self._error = error ; logging.error("^ %s/error %s" % (self.type, str(error)))

    def disconnected(self, *args, **kwargs):
        self._connected.clear()
        self._status = "disconnect"
        logging.warn("^ %s/disconnect %s" % (self.type, self.user))

    def connected(self, *args, **kwargs): 
        self._status = "connected"
        self._connected.set()
        logging.warn("# %s/connected %s" % (self.type, self.user))

    def loop(self, *args, **kwargs): self.client.process()

    def say(self, *args, **kwargs):
        to = args[0]
        txt = str(args[1])
        self.client.send_message(to, txt)
        logging.info("> %s/say %s" % (self.type, to))

    def event(self, *args, **kwargs): return self._queue.get()

    def messaged(self, data, *args, **kwargs):
        logging.debug("< %s/read %s" % (self.type, str(data["data"])))
        m = Object(**data)
        if m.type == "error": logging.error("^ %s" % m.error) ; return
        m.cc = self.cc
        m["from"] = str(m["from"])
        if self.user in m["from"]: logging.info("%s/%s %s" % (self.type, m.type, m["from"])) ; return
        m.origin = m["from"]
        m.channel = m.origin
        m.to = m.origin
        m.element = "message"
        m.txt = m["body"]
        m._target = self
        if '<delay xmlns="urn:xmpp:delay"' in str(data):
            logging.info("# %s/ignore %s %s" % (self.type, m.type, m.origin))
            return
        logging.info("< %s/%s %s" % (self.type, m.type, m.origin))
        self._queue.put_nowait(m)

    def presenced(self, data, *args, **kwargs):
        logging.info("< %s/%s %s" % (self.type, "presence", data["data"]))
        o = Object(data)
        o["from"] = str(o["from"])
        o.origin = o["from"]
        if "txt" not in o: o.txt = ""
        o.element = "presence"
        if o.type == 'subscribe':
            pres = Object({'to': o["from"], 'type': 'subscribed'})
            self.client.send_presence(pres)
            pres = Object({'to': o["from"], 'type': 'subscribe'})
            self.client.send_presence(pres)
        elif o.type == "unavailable":
            if o.origin in self.channels: self.channels.remove(o.origin)
        else:
            if o.origin != self.user and o.origin not in self.channels: self.channels.append(o.origin)
        o.no_dispatch = True
        o._target = self
        logging.info("< %s/%s %s" % (self.type, o.type, o.origin))
        self._queue.put_nowait(o)

    def exit(self, *args, **kwargs):
        self.client.disconnect()
        Bot.exit(self)
        self._queue.put(None)
        
def init(*args, **kwargs):
    cfg = Object().last("cfg", "xmpp")
    if not cfg: cfg = maincfg.xmpp ; cfg.save()
    password = getpass.getpass()
    xmpp = XMPP(password, **cfg)
    fleet.append(xmpp)
    main.put(xmpp.start)
