# clock.py
#
#

""" op de klok geschakeld. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.utils.time import get_day, get_hour, today, short_date
from meds.dispatcher import Dispatcher
from meds.utils.name import named
from meds.object import Object
from meds.main import fleet, main

import threading
import logging
import time

# TIMER

class Timer(Object):

    def __init__(self, time_alarm, event, *args, **kwargs):
        Object.__init__(self, *args, **kwargs)
        self.sleep = time_alarm - time.time()
        self.event = event

    def start(self, *args, **kwargs):
        self._timer = threading.Timer(self.sleep, self.echo)
        self._timer.setDaemon(True)
        self._timer.setName(named(self.echo))
        self._timer._start = time.time()
        self._timer._last = time.time()
        self._timer._status = "start"
        self._timer._status = "ok"
        self._timer.sleep = self.sleep
        logging.warn("# sleeping %s seconds" % self.sleep) 
        self._timer.start()

    def echo(self, *args, **kwargs):
        for bot in fleet: bot.announce(self.event.txt)
        self.event.done = True
        self.event.save()

    def stop(self, *args, **kwargs):
        self._timer._status = "stopped"
        self._timer.cancel()

running = []
        
## LOOPS

class Repeater(Object):

    """ a Repeater calls a function every x seconds. """ 

    def __init__(self, sleep, func, *args, **kwargs):
        """ args = function, sleeptime. """
        Object.__init__(self, **kwargs)
        self.kwargs = kwargs
        self.args = args 
        self.func = func
        self.sleep = sleep
        self._start = time.time()
        self._status = "start"
        self._running = named(self.func)

    def start(self, *args, **kwargs):
        """ initialize a Looper. """
        self._timer = threading.Timer(self.sleep, self.run)
        self._timer.setDaemon(True)
        if "name" in self.kwargs: self._timer.setName(self.kwargs["name"])
        else: self._timer.setName(named(self.func))
        self._timer._nrs = Object()
        self._timer._nrs.loops = 0
        self._timer._start = time.time()
        self._timer._last = time.time()
        self._timer._status = "ok"
        self._timer.sleep = self.sleep
        self._timer.start()

    def run(self, *args, **kwargs):
        self.start()
        self._timer._status = "ok"
        self._timer._nrs.loops += 1
        self.func(**self.kwargs)
 
    def exit(self, *args, **kwargs):
        self._status = ""
        self._timer.cancel()

## COMMANDS

def clock_timer(*args, **kwargs):
    rest = args[0].parsed.rest
    if not rest: return
    day = get_day(rest) or today()
    hour = get_hour(rest)
    target = day + get_hour(rest)
    t = Object()
    t.services = "clock"
    t.prefix = "timer"
    t.txt = rest
    t.time = target
    t.done = False
    t.save()
    timer = Timer(target, t)
    main.put(timer.start)
    args[0].ok()
    logging.warn("# Timer/start %s" % t.json())

main.register("clock.timer", clock_timer)

## INIT

def init():
    for fn in main.all("timer"):
        o = Object().load(fn)
        if time.time() < int(o.time):
            if "time" not in o: continue
            timer = Timer(int(o.time), o)
            main.put(timer.start)
