# meds/dispatcher.py
#
#

""" basic package for the program. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.errors import NoFunction, NoEvent, RemoteDisconnect
from meds.utils.name import naming, named, typed, naming
from meds.utils.trace import get_plugname, get_strace
from meds.utils.parse import txt_parse
from meds.utils.time import elapsed
from meds.errors import error

from meds.object import Object

import collections
import threading
import logging
import random
import queue
import time

## TASKS

class Task(threading.Thread):

    """ the working unit of MEDS.  """

    def __init__(self, *args, **kwargs):
        threading.Thread.__init__(self, *args, **kwargs)
        self._queue = queue.Queue()
        self.setDaemon(True)
        self.result = None
        
    def __iter__(self):
        for x in dir(self): yield x

    ## INPUT

    def put(self, *args, **kwargs):
        """ put arguments/kwargs to the Task. """
        self._last = time.time()
        self._queue.put((args, kwargs))
        return self

    def join(self, *args, **kwargs):
        threading.Thread.join(self)
        return self.result

    ## RUNNING

    def run(self, *args, **kwargs):
        """ loop that executes the (function, ding) pairs. """
        self.type = "Task"
        self._start = time.time()
        self._last = time.time()
        self._status = "ok"
        self.name = ""
        while self._status:
            arglist, kwargdict = self._queue.get()
            if not arglist or not arglist[0]: break
            self.func, *arguments = arglist
            self.name = named(self.func)
            self.setName(self.name)
            try: self.result = self.func(*arguments, **kwargdict)
            except Exception as ex: self._status = ex ; error("", self.name) ; break
            if self._queue.empty(): break
        logging.info("! Task/run %s" % self.name)
        
    def exit(self, *args, **kwargs):
        self._status = ""
        self.put((None, {}))

## DISPATCHER

class Dispatcher(Object):

    """ the dispatcher delegates the workload to the Runners. Runner gets instantiated when needed. """

    cc = "!"
    default = ""

    def __init__(self, *args, **kwargs):
        Object.__init__(self, *args, **kwargs)
        self._input = queue.Queue()
        self._output = queue.Queue()
        self._ready = threading.Event()

    ## INPUT

    def put(self, *args, **kwargs):
        task = Task()
        task.start()
        task.put(*args, **kwargs)
        return task

    ## RESULTS

    def collect(self, *args, **kwargs):
        thrs = args[0]
        if not thrs: return []
        result = []
        for thr in thrs:
            if thr:
                res = thr.join()
                if res: result.append(res)
        return result

    ## STOP

    def stop(self, *args, **kwargs):
        self._status = ""
        self.put(None)

## SHUTDOWN

def shutdown(*args, **kwargs):
    if args: name = args[0]
    else: name = ""
    for thr in threading.enumerate():
        if str(thr).startswith("<_"): continue
        if name and name not in str(thr): continue
        if "stop" in dir(thr): thr.stop()
        elif "exit" in dir(thr): thr.exit()
        elif "cancel" in dir(thr): thr.cancel()
        else: thr.join(0.1)
        logging.warn("# stop %s" % naming(thr))
    return True

