# main.py
#
#

""" main module. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.utils.opts import make_opts, opts_defs
from meds.utils.other import hello, show_eggs
from meds.defines import GREEN, ENDC
from meds.dispatcher import shutdown
from meds.object import Object
from meds.plugins import Plugins
from meds.log import log_init
from meds.cfg import cfg

import logging
import time
import sys
import os

## PATH

def medspath():
    for path in sys.path:
        if "meds" in path and "egg" in path: yield path  

## MAIN 

class Main(Plugins):

    def boot(self, *args, **kwargs):
        """
             Enable runtime services of the MEDS bot.

             >>> main = Main()
             >>> main.boot("MEDS")
             >>> main.wait()

        """ 
        self._start = time.time()
        opts, arglist = make_opts(opts_defs)
        cfg.main.update(vars(opts))
        for key,value in kwargs.items():
            if not getattr(cfg.main , key, ""): cfg.main[key] = value
        if not cfg.main.workdir: cfg.main.workdir = os.path.expanduser("~/.meds")
        cfg.main.runargs = arglist
        log_init(cfg.main.loglevel, colors=cfg.main.colors)
        if cfg.main.shell: hello(*args, **cfg.main)
        if cfg.main.path:
            for path in sys.path: print(path)
            print()
        plugins.boot(cfg.main.packages)
        if "all" in cfg.main.modules: plugins.loaddir(os.getcwd())
        else:
            for mod in cfg.main.modules.split(","):
                if not mod: continue
                plugins.load(mod)
        self.ready()

    def shutdown(self, *args, **kwargs):
        logging.warn("# shutdown")
        plugins.shutdown()
        shutdown()
        self.stop()
        time.sleep(0.01)

## DEFINES

main = Main()
plugins = Plugins()
fleet = []
