# plugins.py
#
#

""" meds plugins. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.dispatcher import Dispatcher
from meds.utils.name import named
from meds.object import Object
from meds.main import shutdown
from meds.errors import error
from meds.cfg import cfg

import importlib
import logging
import pkgutil
import os

## PLUGINS

class Plugins(Dispatcher):

    """ register plugins. """

    plugnames = ""

    ## INIT

    def init(self, *args, **kwargs):
        """ initialize plugins, """
        plugnames = args[0]
        modnames = plugnames.split(",") 
        for modname in sorted(modnames, reverse=True):
            if not modname: continue
            objs = self.find(modname)
            for obj in objs:
                logging.warn("# %s/init %s" % (self.type, modname))
                if modname == "xmpp": obj.init() ; continue
                init = getattr(obj, "init", None)
                if init: init()
        self.ready()

    def boot(self, *args, **kwargs):
        from meds.cfg import cfg
        packages = args[0]
        for package in packages:
            if package in cfg.main.exclude and package.split(",")[-1] not in cfg.main.modules:
                logging.warn("# Plugins/ignore %s" % package)
                continue
            mod = self.load(package)
            register = getattr(mod, "register", None)
            if register:
                logging.warn("# %s/register %s" % (self.type, package))
                register()
            init = getattr(mod, "init", None)
            if init:
                logging.warn("# %s/init %s" % (self.type, package))
                init()

    def shutdown(self, *args, **kwargs):
        for name, obj in self.items():
            if "shutdown" in dir(obj):
                logging.warn("# %s/shutdown" % self.type)
                obj.shutdown()

    ## SEARCH

    def search(self, *args, **kwargs):
        """ find matching name. """
        n = args[0]
        for name in self.names():
            if n in name: yield name

    ## LOAD

    def load(self, *args, **kwargs):
        """ load plugins from the plugin directory. """
        from meds.cfg import cfg
        modname = args[0]
        logging.warn("# %s/load %s" % (self.type, modname))
        self[modname] = mod = self.loading(modname)
        if "__path__" in dir(mod):
            for pkg in pkgutil.walk_packages(mod.__path__, modname + "."):
                name = pkg[1]
                self[name] = self.loading(name)
        return self[modname]

    def loading(self, name, package=None, force=True):
        """ load module. """
        logging.debug("! %s/load %s" % (self.type, name))
        if name in self and not force: mod = self.reload(name)
        else: mod = importlib.import_module(name, package)
        return mod

    def loaddir(self, *args, **kwargs):
        dirname = args[0]
        for fn in os.listdir(dirname):
            if not fn.endswith(".egg"): continue
            name = fn.split("-")[0]
            try: self[name] = self.loading(name)
            except Exception as ex: print(ex)

    ## RELOAD

    def reload(self, *args, **kwargs):
        name = args[0]
        logging.warn("# %s/reload %s" % (self.type, name))
        self[name] = importlib.reload(self[name])
        return self[name]

    def reloading(self, *args, **kwargs):
        names = self.search(args[0])
        for name in names: self.reload(name)

## CMNDS

def plugins_stop(event):
    from meds.main import plugins
    name = event.parsed.rest
    if name and plugins.search(name): shutdown(name) ; event.ok(name)
    logging.warn("# Plugins/stop %s" % name)

def plugins_start(event):
    from meds.main import plugins
    args = event.parsed.args
    if args:
        pluginnames = plugins.search(*args)
        for pname in pluginnames:
            plugins.load(pname)
            plugins.init(event.parsed.args[0])
            event.ok(pname)
    logging.warn("# Plugins/start %s" % ", ".join(pluginnames))

def plugins_reload(event):
    from meds.main import plugins
    result = plugins.reloading(*event.parsed.args)
    if result: event.ok(named(result))
    logging.warn("# Plugins/reload")

def plugins_load(event):
    from meds.main import plugins
    name = event.parsed.rest
    modnames = plugins.search(name)
    for modname in modnames:
        try: event.ok(plugins.load(modname))
        except Exception as ex: event.reply(str(ex))
    logging.warn("# Plugins/load")

## REGISTER

def register(*args, **kwargs):
    from meds.main import main
    main.register("plugins.reload", plugins_reload)
    main.register("plugins.load", plugins_load)
    main.register("plugins.start", plugins_start)
    main.register("plugins.stop", plugins_stop)
