# db.py
#
#

""" storage on disk. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.main import main
from meds.object import Object

import logging

## CMNDS

def db_rm(event):
    try: key, match = event.parsed.rest.split(" ", 1)
    except ValueError: return
    nr = 0
    for fn in main.all(key):
        obj = Object().load(fn)
        if key not in obj: continue
        if match not in obj[key]: continue
        obj.deleted = True
        obj.sync()
        nr += 1
    event.ok(nr)
    logging.warn("# Db/rm %s" % nr)

main.register("db.rm", db_rm)

def db_restore(event):
    try: key, match = event.parsed.args
    except ValueError: return
    nr = 0
    for fn in main.all(*event.parsed.args):
        obj = Object().load(fn)
        if key not in obj: continue
        if match not in obj[key]: continue
        if "deleted" not in obj: continue
        obj.deleted = False
        obj.sync()
        nr += 1
    logging.warn("# Db/restore %s" % nr)
    event.ok(nr)

main.register("db.restore", db_restore)

def db_find(event):
    nr = 1
    objs = sorted(event.selected(event.parsed), key=lambda x: x.timed())
    for obj in objs: event.reply("%s %s" % (nr, obj.display(event.parsed.args))) ; nr += 1
    logging.warn("# Db/find %s" % nr)

main.register("db.find", db_find)

def db_dump(event):
    objs = sorted(event.selected(event.parsed), key=lambda x: x.timed())
    for obj in objs: event.reply(obj.json())
    logging.warn("# Db/dump %s" % len(objs))

main.register("db.dump", db_dump)

def db_all(event):
    if event.parsed.args: search_term = event.parsed.args[0]
    else: search_term = ""
    keys = []
    nr = 0
    for fn in main.all():
        obj = Object().load(fn)
        for key in obj:
            if search_term in str(obj[key]) and key not in keys: keys.append(key) ; nr += 1
    if keys: event.reply(", ".join(keys))
    logging.warn("# Db/all %s" % nr)

main.register("db.all", db_all)

def db_deleted(event):
    if not event.parsed.args: return
    key = event.parsed.args[0]
    nr = 0
    for fn in main.all(key):
       obj = Object().load(fn) 
       if "deleted" not in obj: continue
       event.reply(obj.get(key, ""))
       nr += 1
    logging.warn("# Db/deleted %s" % nr)

main.register("db.deleted", db_deleted)

def db_first(event):
    event.reply(main.first(*event.parsed.args))
    logging.warn("# Db/first")

main.register("store.first", db_first)

def db_last(event):
    event.reply(main.last(*event.parsed.args))
    logging.warn("# Db/last")

main.register("db.last", db_last)
