# meds/dutch/udp.py
#
#

""" relay txt through a udp port listener. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from meds.errors import error
from meds.utils.name import name
from meds.extra import rijndael
from meds.main import main
from meds.cfg import cfg
from meds.object import Object

import logging
import socket
import time

crypt = rijndael.rijndael(cfg.udp.seed)

## UDP class

class UDP(Object):

    def __init__(self, *args, **kwargs):
        Object.__init__(self, *args, **kwargs)
        self._status = "start"
        self._start = time.time()
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try: self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        except: pass
        self._sock.setblocking(1)

    def start(self):
        self._sock.bind((cfg.udp.host, cfg.udp.port))
        logging.warn("# start %s/%s" % (cfg.udp.host, cfg.udp.port)) 
        self.ready()
        self._status = "ok"
        while self._status:
            input, addr = self._sock.recvfrom(64000)
            if not self._status: break
            input = str(input.rstrip(), "utf-8")
            data = ""
            for i in range(int(len(input)/16)):
                txt = input[i*16:i*16+16]
                try: data += crypt.decrypt(txt)
                except Exception as ex: error() ;  break
            if not data: break
            self.output(data, addr)
        
    def exit(self, *args, **kwargs):
        logging.warn("# exit %s" % name(self))
        self._status = "exit"
        self._sock.settimeout(1.0)
        self._sock.sendto(bytes("bla", "utf-8"), (cfg.udp.host, cfg.udp.port))
        #self._sock.shutdown(socket.SHUT_WR)

    def output(self, input, addr):
        passwd, text = input.split(" ", 1)
        text = text.replace("\00", "")
        if passwd == cfg.udp.password: main.announce(text)

def init(*args, **kwargs):
    udp = UDP()
    main.put(udp.start)
