# test.py
#
#

""" test plugin. """

## IMPORTS

from meds.utils.name import typed
from meds.main import main
from meds.bots import TestBot
from meds.object import Object
from meds.cfg import cfg

import logging
import time

## DEFINES

exclude = ["fetch", "test"]

## COMMANDS

def test_forcedreconnect(event):
    """ no arguments - do a forced reconnect. """
    bot = event._target
    try: bot._sock.shutdown(2)
    except: event.reply("%s bot doesn't have a _sock attribute" % typed(bot))
    logging.warn("# Test/forcedreconnect")

main.register('test.forcedreconnect', test_forcedreconnect)

def test_forcedexception(event):
    """ no arguments - raise a exception. """
    event.reply("raising exception.")
    raise Exception('test exception')
    logging.warn("# Test/forcedexception")

main.register('test.forcedexception', test_forcedexception)

def test_wrongxml(event):
    """ no arguments - try sending borked xml. """
    event.reply('sending bork xml')
    bot = event._target
    bot.raw('<message asdfadf/>')
    logging.warn("# Test/wrongxml")

main.register('test.wrongxml', test_wrongxml)

def test_xmlescape(event):
    """ no arguments - try sending borked xml. """
    event.reply('sending xml')
    bot = event._target
    event.reply('<message asdfadf/>')
    logging.warn("# Test/xmlescape")

main.register('test.xmlescape', test_xmlescape)

def test_unicode(event):
    """ no arguments - send unicode test down the output paths. """
    outtxt = u"ÄÃ­Å¥ Ã¬Å¡ Ã©Ã¨Ã± Ã«ÅÄÃ¸ÄÄ±ÅÄÅ§ÄÅÅ£Â· .. ã«ã»ãããã¯ãªãã¾ãã .. â0â°â1Â¹â2Â²â3Â³â4â´â5âµâ6â¶â7â·â8â¸â9â¹ .. âââââââââââââââ .. .. uÇÊoqÇÊsÉ¹Çpuo pÉdÄ± ÇÉ¾ ÊpnoÉ¥ ÇÉ¾"
    event.reply(outtxt)
    logging.warn("# Test/unicode")

main.register('test.unicode', test_unicode)

def test_deadline(event):
    """ no arguments - slee for 40 seconds in the mainloop. """
    try: nrseconds = int(event.parsed.rest)
    except: event.reply("need number of seconds to sleep.") ; return
    event.reply('starting %s sec sleep' % nrseconds)
    time.sleep(nrseconds)
    logging.warn("# Test/deadline")

main.register('test.deadline', test_deadline)

def test_html(event):
    """ arguments: [<txt>] - test the html=True option to event.reply(). """
    event.reply('<span style="font-family: fixed; font-size: 10pt"><b>YOOOO BROEDERS</b></span>')
    logging.warn("# Test/html")

main.register('test.html', test_html)

def test_cmnds(event):
    if not main.allowed(event.origin): event.reply("ENOTALLOWED %s" % event.origin) ; return
    bot = TestBot()
    bot.workdir = "test.data"
    try: nrloops = int(event.parsed.rest)
    except: nrloops = 1
    for x in range(nrloops):
        for cmnd in sorted(list(main.names())):
            go = True
            for c in exclude:
                if c in cmnd: go = False
            if not go: continue
            e = bot.event("!" + cmnd + " cfg")
            logging.warn("< %s" % e.txt)
            event.reply("cmnds is %s" % e.txt)
            bot.dispatch(e, nocollect=True)
    logging.warn("# Test/cmnds")

main.register("test.cmnds", test_cmnds)
